%in this version the raw data is not white balanced each time but always
%projected with the original matrix and wb multipliers 
wl = 380:730;

load matlab.mat         %instead of files below

% illType = 'D';
% illcct = 5300;                                      %illuminant CCT
% ill = illuminant([illType num2str(illcct)],wl)/100; %needs optprop
% cc24 = xlsread('E:\My Drive\Camera\Data\Reflectance\CC24_Classic_BabelColor30_Pre2015.csv','H2:AQ25')';
% cc24 = interp1(380:10:730,cc24,wl);                             %reflectance
% cmf = csvread('E:\My Drive\Camera\Data\CMF\ciexyz31_1.csv');    %1931 Obs
% cmf = cmf(21:371,2:4);                                          %380:730nm
% ssf = csvread('E:\My Drive\Camera\Data\CFA\D5100.csv');         %Nikon D5100
% ssf = interp1(380:5:780,ssf(:,2:4),wl);                         %Darrodi et al
%%
plotFlag = 0;                                       %illuminant sweep if set
wbFlag = 0;                                         %wb each time if set
                                                    %else use original wb
xyzn = ill * cmf;                                   %illuminant neutral
xyzn = xyzn/xyzn(2);                                %illuminant White Point
cctDuv(xyzn);                                       %check

Ln = cc24 .* ill';                                  %radiance from CC24 target

xyzRef = Ln' * cmf;                                 %reference values for given ill
xyzRef = xyzRef/xyzRef(22,2)*mean(cc24(:,22))/100;  %normalize to reference
labRef = xyz2labn(xyzRef,xyzn);

grey = sum(ssf .* ill');                            %camera neutral
wbMult0 = 1 ./ grey;                                %Wb multipliers
wbMult0 = wbMult0/wbMult0(2);                       %normalized

raw = Ln' * ssf;                                    %raw data from radiance
wbraw = raw .* wbMult0;                             %white balance raw data
wbraw = wbraw/wbraw(22,2)*xyzRef(22,2);             %normalize to reference

[M,M0,de76,de00,k,output] = estimateMatrixWpp(wbraw,xyzRef,xyzn,'fminunc');
%xyz = wbraw * M * k format.  Remember: Forward Matrix is M' in this case
disp(['M0 de00: ' num2str( mean(deltaE00( xyz2labn(wbraw*M0,xyzn), labRef )))])

%
M'*diag(wbMult0)*[grey/grey(2)*0.18*15500]'
ans/ans(2)
cctDuv(ans');
%% - see what happens if illuminant changes but all else stays the same
if plotFlag                                         %#ok<*UNRCH>
    cctRange = illcct-800:100:illcct+1200;
    mn = zeros(length(cctRange),1);
    mx = mn; st = mn; mxP = mn; cct = mn; duv = mn; cctErr = mn;
    fprintf('%s Illuminant, Err:   de00 mean     max    stdv   | de76 mean     max    stdv\n', illType)
    i = 1;
    for T = cctRange
        ill = illuminant([illType num2str(T)], wl); %vary illuminant
        Ln = cc24 .* ill';                          %radiance from CC24 under illuminant
        
        %raw section: camera readings in the field, always available
        raw = Ln' * ssf;                            %raw data from radiance
        grey = sum(ssf .* ill');                    %camera neutral: gray card reading
        grey = grey / grey(2);                      %changes with illuminant
                                                    
        if wbFlag
            wbraw = raw ./ grey;                    %white balance each time
            WPest = sum(M);                         %illuminant neutral WP in xyz
            wbTit = '';
        else
            wbraw = raw .* wbMult0;                 %use original wb multipliers
            WPest = grey.*wbMult0*M;                %illuminant neutral WP in xyz
            wbTit = 'non-';
        end
        WPest = WPest / WPest(2);                   %Estimated White Point
        wbraw = wbraw/wbraw(22,2)*xyzRef(22,2);     %normalize
        
        xyz = wbraw * M;
        
        %reference section: meter readings in the field, mostly not available
        xyzn = ill * cmf;                           
        xyzn = xyzn/xyzn(2);                        %Actual WP
        
        xyzRef = Ln' * cmf;                         %reference values for given ill
        xyzRef = xyzRef/xyzRef(22,2)*mean(cc24(:,22))/100;
        labRef = xyz2labn(xyzRef,xyzn);
        
%         de00 = deltaE00( xyz2labn(xyz,xyzn), labRef );
%         de76 = deltaE76( xyz2labn(xyz,xyzn), labRef );
%         
%         patches = [1:18 22];                        %only one neutral patch in calcs
%         
%         mn(i) = mean(de00(patches));
%         [mx(i),mxP(i)] = max(de00(patches));
%         st(i) = std(de00(patches));
        
        [cct(i),duv(i)] = cctDuv(WPest,0);
        cctErr(i) = deltaE00(xyz2labn(WPest,xyzn),xyz2labn(xyzn,xyzn));
        
%         fprintf('%u / %.0f, %.2f:     %7.4f %7.4f %7.4f   |   %7.4f %7.4f %7.4f\n',...
%             T,cct(i),cctErr(i),mn(i),mx(i),st(i),mean(de76),max(de76),std(de76))
        i = i + 1;
    end
    figure
    plot(cctRange,cct,'.k'); hold on
    plot(cctRange,cctRange,'Color',[0.7 0.7 0.7])
    ylabel('Estimated WP CCT (deg K)')
    yyaxis right
    plot(cctRange,cctErr); hold on
    plot(cctRange,duv*1000); grid, grid minor
    xlabel('Reference Illuminant CCT (deg K)');
    ylabel('WP Difference (dE00) / Duv (m)')
    plot(5300,duv(9)*1000,'.k')
    legend('Estimated WP','Actual WP','WP Difference','mDuv');
    title(['White Point Error with Fixed ' illType num2str(illcct) '-CC24 Matrix'])
%     mxP(mxP>18) = 22;                               %see patches above
%     figure; plot(cctRange,mn,cctRange,mx, cctRange,st); grid; grid minor
%     ylim([0 4]); legend('Mean','Max','StdDev')
%     xlabel('Illuminant Correlated Color Temperature'); ylabel('deltaE00')
%     title(['BabelColor30 CC24, ' wbTit 'WB Raw Data, ' illType num2str(illcct) ' Matrix'])
%     text(cctRange,mx,char(num2str(mxP)),'FontSize',8,'VerticalAlignment','bottom')
    %     figure; bar3c(reshape(de00,6,[])',roo2disp(colorchecker))%requires optprop
        %     zlabel('\DeltaE_{00}'); zlim([0 4])
end